; interrupt service routines





	.org	0x2003		;int0 interrupt
	ljmp	int0_isr

	.org	0x200B		;timer0 interrupt
	ljmp	timer0_isr

	.org	0x2013		;int1 interrupt
	ljmp	int1_isr

	.org	0x2023		;uart interrupt
	clr	ri
	clr	ti
	reti



	;int0 is connected to the xilinx chip.  There are
	;many possible interrupt sources that could generate
	;this interrupt
int0_isr:
	push	psw
	push	dpl
	push	dph
	mov	psw, #00001000b		;set reg bank1 (0x08 to 0x0F)
	mov	r7, a
int0_loop:
	mov	dptr, #irq_ident
	movx	a, @dptr
	jb	acc.0, int0_ide_dma
	jb	acc.1, int0_mp3_dma

	;this shouldn't happen, as we're not using hardware memcpy yet
	mov	dptr, #irq_memcpy_ack
	movx	@dptr, a
	mov	a, r7
	pop	dph
	pop	dpl
	pop	psw
	ret


	;when we get here, a DMA transfer from the IDE drive has
	;been completed
int0_ide_dma:
	mov	dptr, #irq_dma_ide_ack
	movx	@dptr, a

	;need to clear the transfer pending bit

	;jump into int1, to start another IDE command
	ajmp	int1_start_new_cmd



	;when we get here, the STA013 chip is ready for more data
int0_mp3_dma:
	mov	dptr, #irq_dma_mp3_ack
	movx	@dptr, a
	mov	a, play_req_tail
	cjne	a, play_req_head, play_next_req
	mov	a, #255	
        mov     clust_rreq_head, a      ;no play requests pending
        mov     clust_rreq_tail, a
	sjmp	int0_loop
play_next_req:
        inc     a
        cjne    a, #play_req_size, play_next_req2
        clr     a
play_next_req2:
        mov     play_req_tail, a

	;now this play_request is effectively removed from the queue,
	;so we'd better grab it into registers
        rlc     a			;sizeof(struct play_request) == 4
        rlc     a
	anl	a, #11111100b
        add     a, #play_req & 255
        mov     dpl, a
        clr     a
        addc    a, #play_req >> 8
        mov     dph, a                  ;now DPTR points to pending request
        movx    a, @dptr
        mov     r2, a           ;read "block_to_play" into r2/r3
	inc	dptr
        movx    a, @dptr
        mov     r3, a
	inc	dptr
        movx    a, @dptr
        mov     r4, a		;read "num_bytes_to_play" into r4/r5
	inc	dptr
        movx    a, @dptr
        mov     r5, a

        ;now let's map the block with the MP3 data to 0x9000
        mov     dptr, #dram_page_cfg + (0x09 * 2)
        mov     a, r2
        movx    @dptr, a
        inc     dptr
        mov     a, r3
        movx    @dptr, a

        ;finally, we're done with the memory management stuff, and
        ;we can now set up the DMA transfer and start it.
        mov     dptr, #dma_mp3_src
        clr     a
        movx    @dptr, a                ;write LSB of src addr
        inc     dptr
        mov     a, #0x90
        movx    @dptr, a                ;write MSB of src addr
        mov     dptr, #dma_mp3_count
        mov     a, r4
        movx    @dptr, a                ;write LSB of length
        inc     dptr
        mov     a, r5
        movx    @dptr, a                ;write MSB of length
        mov     dptr, #dma_mp3_go
        movx    @dptr, a                ;begin the transfer

	mov	a, r7
	pop	dph
	pop	dpl
	pop	psw
	ret














	;int1 is connected to the IDE drive... when we get an
	;interrupt, the drive has finished a command.
int1_isr:
	push	psw
	push	dpl
	push	dph
	mov	psw, #00001000b		;set reg bank1 (0x08 to 0x0F)
	mov	r7, a

	;we might need to start a new command, or we might be in the
	;middle of a read request and the drive is now ready to begin
	;transfering data, either way we need to read the IDE status
	;register to clear the interrupt and find out what's up.

	mov	dptr, #ide_status
	movx	a, @dptr




int1_start_new_cmd:
        mov     a, fat_rreq_tail
        cjne    a, fat_rreq_head, ide_next_high
        mov     a, #255
        mov     fat_rreq_head, a        ;no high priority requests
        mov     fat_rreq_tail, a
        mov     a, clust_rreq_tail
        cjne    a, clust_rreq_head, ide_next_low
        mov     a, #255
        mov     clust_rreq_head, a      ;no low priority requests
        mov     clust_rreq_tail, a
	mov	a, r7
	pop	dph
	pop	dpl
	pop	psw
        ret

        ;there's a high priority request pending.
ide_next_high:
        inc     a
        cjne    a, #fat_rreq_size, ide_next_high2
        clr     a
ide_next_high2:
        mov     fat_rreq_tail, a
        swap    a
        rr      a               ;sizeof(struct read_request) == 8
        add     a, #fat_read_req & 255
        mov     dpl, a
        clr     a
        addc    a, #fat_read_req >> 8
        mov     dph, a
        sjmp    ide_next_cmd2

        ;there's a low priority request pending.
ide_next_low:
        inc     a
        cjne    a, #clust_rreq_size, ide_next_low2
        clr     a
ide_next_low2:
        mov     clust_rreq_tail, a
        mov     b, #8           ;sizeof(struct read_request) == 8
        mul     ab
        add     a, #clust_read_req & 255
        mov     dpl, a
        mov     a, b
        addc    a, #clust_read_req >> 8
        mov     dph, a

        ;when we get here, the request has already been
        ;removed from its queue, but we still need to
        ;copy its data (@dptr).
ide_next_cmd2:
        movx    a, @dptr
        mov     r3, a           ;read LBA (source) into r3/r6
        inc     dptr
        movx    a, @dptr
        mov     r4, a
        inc     dptr
        movx    a, @dptr
        mov     r5, a
        inc     dptr
        movx    a, @dptr
        mov     r6, a
        inc     dptr
        movx    a, @dptr
        mov     r1, a           ;read block (destination) into r1/r2
        inc     dptr
        movx    a, @dptr
        mov     r2, a
        inc     dptr
        movx    a, @dptr
        mov     r0, a		;read request type into r0




	mov	a, r7
	pop	dph
	pop	dpl
	pop	psw
	ret










timer0_isr:
	push	psw
	push	dpl
	push	dph
	mov	psw, #00001000b		;set reg bank1 (0x08 to 0x0F)
	mov	r7, a


	mov	a, r7
	pop	dph
	pop	dpl
	pop	psw
	ret
